/*----------------------------------------------------------------------------
miniAudicle
Cocoa GUI to chuck audio programming environment

Copyright (c) 2005 Spencer Salazar.  All rights reserved.
http://chuck.cs.princeton.edu/
http://soundlab.cs.princeton.edu/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// file: NumberedTextView.m
// desc: view class for implementing a text view with scrollers and line numbers
//       Based on Koen van der Drift's MyTextView class.  
//
// author: Spencer Salazar (ssalazar@princeton.edu)
// date: Spring 2006
//-----------------------------------------------------------------------------

#import <Cocoa/Cocoa.h>
#import "mASyntaxHighlighter.h"

@interface NumberedTextView : NSView
{
    BOOL drawNumbersInMargin;
    BOOL drawLineNumbers;
    BOOL enable_syntax_highlighting;
    NSMutableDictionary * marginAttributes;
    NSTextView * text_view;
    
    IDEKit_LexParser * syntax_highlighter;
    id colorer;
    
    // code editor data
    BOOL open_bracket;
    int close_bracket;
    BOOL use_tabs;
    int tab_width;
    NSRange changed_range;
}

-(void)initLineMargin:(NSRect)frame;

- (void)textStorageDidProcessEditing:(NSNotification *)notification;
- (BOOL)textView:(NSTextView *)aTextView doCommandBySelector:(SEL)aSelector;
- (BOOL)textView:(NSTextView *)aTextView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString;
- (void)textDidEndEditing:(NSNotification *)n;

- (void)initSyntaxHighlighting;
- (void)setSyntaxHighlighter:(IDEKit_LexParser *)sh colorer:(id)c;
- (void)syntaxColoringChanged:(NSNotification * )n;

- (void)updateMargin;
- (void)updateLayout;

- (void)drawEmptyMargin:(NSRect)aRect;
- (void)drawNumbersInMargin:(NSRect)aRect;
- (void)drawOneNumberInMargin:(unsigned)aNumber inRect:(NSRect)aRect;

- (NSRect)marginRect;

- (NSTextView *)textView;

@end
