/*----------------------------------------------------------------------------
miniAudicle
Cocoa GUI to chuck audio programming environment

Copyright (c) 2005 Spencer Salazar.  All rights reserved.
http://chuck.cs.princeton.edu/
http://soundlab.cs.princeton.edu/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// file: miniAudicleDocument.mm
// desc: Document class, creates a new window for each new document and manages
//       document-level connections to miniAudicle
//
// author: Spencer Salazar (ssalazar@princeton.edu)
// date: Autumn 2005
//-----------------------------------------------------------------------------

#import "miniAudicleDocument.h"
#import "miniAudicleController.h"
#import "KBPopUpToolbarItem.h"

#include "chuck_parse.h"

#define USE_POPUP_TOOLBAR_ITEMS 0

@interface NSString ( mADocument )
- (string)stlString;
@end

@implementation NSString ( mADocument )

- (string)stlString
{
    NSData * data = [self dataUsingEncoding:NSASCIIStringEncoding
                       allowLossyConversion:YES];
    return string( ( char * ) [data bytes], [data length] );
}

@end

//@interface mAText

@implementation miniAudicleDocument

- (id)init
{
	if( self = [super init] )
	{
		ma = nil;
		text_view = nil;
		toolbar = nil;
		status_text = nil;
		data = nil;
		
		docid = 0;
		vm_on = FALSE;
	}
    
    return self;
}

- (void)makeWindowControllers
{
	NSRect content_rect = { NSZeroPoint, NSMakeSize( 600, 500 ) };
	unsigned int window_style = NSTitledWindowMask | NSClosableWindowMask | 
		NSResizableWindowMask | NSMiniaturizableWindowMask;
	NSWindow * window = [[NSWindow alloc] initWithContentRect:content_rect
													styleMask:window_style
													  backing:NSBackingStoreBuffered
														defer:YES];
	[window autorelease];
	
	
	miniAudicleController * mac = [NSDocumentController sharedDocumentController];
	[mac setLastWindowTopLeftCorner:[window cascadeTopLeftFromPoint:[mac lastWindowTopLeftCorner]]];
		
	// retrieve the frame of the content area in the window
	content_rect = [[window contentView] frame];
	
	NSView * content_view = [[[NSView alloc] initWithFrame:content_rect] autorelease];
	
	content_rect.origin.y += 20;
	content_rect.size.height -= 20;
	
	// create a text view, adjust it for display
	text_view = [[NumberedTextView alloc] initWithFrame:content_rect];
    [text_view setSyntaxHighlighter:[mac syntaxHighlighter] colorer:mac];
    if( data != nil )
		[[text_view textView] setString:data];

    [text_view setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];

	// embed the text view in the content area
	[content_view addSubview:text_view];
	
	content_rect = NSMakeRect( 0, 0, 600, 20 );
	
	// make status text view 
	status_text = [[NSTextField alloc] initWithFrame:content_rect];
	
	[status_text setBackgroundColor:[NSColor gridColor]];
	[status_text setAutoresizingMask:NSViewWidthSizable];
	[status_text setBezeled:NO];
	[status_text setEditable:NO];
	[status_text setFont:[NSFont systemFontOfSize:[NSFont smallSystemFontSize]]];
	
	[content_view addSubview:status_text];
	
	// embed primary view in the window's content area
	[window setContentView:content_view];
		
	// build the toolbar
	toolbar = [[NSToolbar alloc] initWithIdentifier:@"miniAudicle"];
	[toolbar setVisible:YES];
	[toolbar setDelegate:self];
	
	// add toolbar to the window
	[window setToolbar:toolbar];
    /*
    // build the shred drawer
    NSTableView * shred_table = [[NSTableView alloc] init];
    
    [shred_table setGridStyleMask:NSTableViewSolidVerticalGridLineMask | NSTableViewSolidHorizontalGridLineMask];
    [shred_table setAutoresizingMask:NSViewHeightSizable];
    //[shred_table set
    
    NSTableColumn * table_column = [[NSTableColumn alloc] initWithIdentifier:@"shred"];
    [table_column autorelease];
    [[table_column headerCell] setStringValue:@"shred"];
    [table_column setWidth:50];
    [table_column setMaxWidth:50];
    [table_column setMinWidth:50];
    [shred_table addTableColumn:table_column];
    
    table_column = [[NSTableColumn alloc] initWithIdentifier:@"time"];
    [table_column autorelease];
    [[table_column headerCell] setStringValue:@"time"];
    [table_column setWidth:80];
    [table_column setMaxWidth:80];
    [table_column setMinWidth:80];
    [shred_table addTableColumn:table_column];
    
    table_column = [[NSTableColumn alloc] initWithIdentifier:@"remove"];
    [table_column autorelease];
    [[table_column headerCell] setStringValue:@"-"];
    [[table_column headerCell] setAlignment:NSCenterTextAlignment];
    [table_column setWidth:18];
    [table_column setMaxWidth:18];
    [table_column setMinWidth:18];
    [shred_table addTableColumn:table_column];
    
    table_column = [[NSTableColumn alloc] initWithIdentifier:@"replace"];
    [table_column autorelease];
    [[table_column headerCell] setStringValue:@"="];
    [[table_column headerCell] setAlignment:NSCenterTextAlignment];
    [table_column setWidth:18];
    [table_column setMaxWidth:18];
    [table_column setMinWidth:18];
    [shred_table addTableColumn:table_column];
    
    NSScrollView * scroll_view = [[NSScrollView alloc] initWithFrame:NSMakeRect( 0, 0, 189, 492)];
    [scroll_view autorelease];
    
    [scroll_view setDocumentView:shred_table];
    [scroll_view setAutoresizingMask:NSViewHeightSizable | NSViewWidthSizable];
    [scroll_view setHasVerticalScroller:YES];
    [scroll_view setHasHorizontalScroller:NO];
    [[scroll_view contentView] setFrame:NSMakeRect( 0, 0, 170, 492)];
    [[scroll_view verticalScroller] setControlSize:NSSmallControlSize];
    
    NSView * shred_view = [[NSView alloc] initWithFrame:NSMakeRect( 0, 0, 191, 492)];
    [shred_view autorelease];
    [shred_view addSubview:scroll_view];
    
    shred_drawer = [[NSDrawer alloc] initWithContentSize:NSMakeSize( 191, 20 )
                                           preferredEdge:NSMinXEdge];
    
    [shred_drawer setMaxContentSize:NSMakeSize( 191, 400 )];
    [shred_drawer setMinContentSize:NSMakeSize( 191, 50 )];
    [shred_drawer setTrailingOffset:15];
    [shred_drawer setLeadingOffset:15];
    [shred_drawer setContentView:shred_view];
    [shred_drawer setParentWindow:window];
    //[shred_drawer open];
    
    //[document_drawer setParentWindow:window];
    //[document_drawer open];
    */
	NSWindowController * window_controller = [[NSWindowController alloc] initWithWindow:window];
	[window_controller autorelease];
	
	[self addWindowController:window_controller];
}

- (void)dealloc
{
	if( text_view != nil)
		[text_view release];
	if( toolbar != nil )
		[toolbar release];
	if( status_text != nil )
		[status_text release];
	if( data != nil )
		[data release];
	if( ma != nil )
        ma->free_document_id( docid );
    
	[super dealloc];
}

- (NSData *)dataRepresentationOfType:(NSString *)type
{
    return [[[text_view textView] string] dataUsingEncoding:NSASCIIStringEncoding
                                       allowLossyConversion:YES];
}

- (BOOL)loadDataRepresentation:(NSData *)t_data ofType:(NSString *)type
{
	data = [[NSString alloc] initWithData:t_data encoding:NSASCIIStringEncoding];
	if( text_view != nil )
		[[text_view textView] setString:data];
    return YES;
}

- (void)setMiniAudicle:(miniAudicle *)t_ma
{
	ma = t_ma;
    docid = ma->allocate_document_id();
}

- (void)add:(id)sender
{
	string result;
    t_CKUINT shred_id;
	string code_name = string( [[self displayName] stlString] );

	string code = [[[text_view textView] string] stlString];
	
	if( ma->run_code( code, code_name, docid, shred_id, result ) )
    {
        [status_text setStringValue:@""];
#if defined( USE_POPUP_TOOLBAR_ITEMS ) && USE_POPUP_TOOLBAR_ITEMS
        NSString * menu_title = [NSString stringWithFormat:@"%u (%@)", 
            shred_id, [self displayName]];
        NSMenuItem * menu_item = [[NSMenuItem alloc] initWithTitle:menu_title
                                                            action:@selector(removeShred:)
                                                     keyEquivalent:@""];
        [menu_item autorelease];
        [menu_item setTag:shred_id];
        [remove_menu insertItem:menu_item atIndex:( [remove_menu numberOfItems] - 2 )];

        menu_item = [[NSMenuItem alloc] initWithTitle:menu_title
                                               action:@selector(replaceShred:)
                                        keyEquivalent:@""];
        [menu_item autorelease];
        [menu_item setTag:shred_id];
        [replace_menu insertItem:menu_item atIndex:( [replace_menu numberOfItems] - 2 )];
#endif
    }
    
    else
        [status_text setStringValue:[NSString stringWithCString:result.c_str()]];
    //miniAudicleController * mac = [NSDocumentController sharedDocumentController];
    //[mac updateSyntaxHighlighting];
}


- (void)remove:(id)sender
{
    string result;
    t_CKUINT shred_id;
    
    if( ma->remove_code( docid, shred_id, result ) )
    {
        [status_text setStringValue:@""];
#if defined( USE_POPUP_TOOLBAR_ITEMS ) && USE_POPUP_TOOLBAR_ITEMS
        [remove_menu removeItem:[remove_menu itemWithTag:shred_id]];
        [replace_menu removeItem:[replace_menu itemWithTag:shred_id]];
#endif
    }
    
    else
        [status_text setStringValue:[NSString stringWithCString:result.c_str()]];
}

- (void)replace:(id)sender
{
    string result;
    t_CKUINT shred_id;
    string code = [[[text_view textView] string] stlString];
    string code_name = [[self displayName] stlString];
    
    if( ma->replace_code( code, code_name, docid, shred_id, result ) )
    {
        [status_text setStringValue:@""];
#if defined( USE_POPUP_TOOLBAR_ITEMS ) && USE_POPUP_TOOLBAR_ITEMS
        [[remove_menu itemWithTag:shred_id] setTitle:[self displayName]];
        [[replace_menu itemWithTag:shred_id] setTitle:[self displayName]];
#endif
    }
    
    else
        [status_text setStringValue:[NSString stringWithCString:result.c_str()]];
    
    //miniAudicleController * mac = [NSDocumentController sharedDocumentController];
    //[mac updateSyntaxHighlighting];
}

- (void)removeall:(id)sender
{
	string result;
	ma->removeall( result );
	[status_text setStringValue:[NSString stringWithCString:result.c_str()]];
}

- (void)removelast:(id)sender
{
	string result;
	ma->removelast( result );
	[status_text setStringValue:[NSString stringWithCString:result.c_str()]];
}

- (void)removeShred:(id)sender
{
    if( [sender tag] == 0 )
    {
        
    }
    
    else
    {
        
    }
}

- (void)replaceShred:(id)sender
{
    
}

- (void)vm_on
{
	[self setVMOn:YES];
}

- (void)vm_off
{
	[self setVMOn:NO];
}

- (void)setVMOn:(BOOL)t_vm_on
{
	vm_on = t_vm_on;
	
	NSArray * toolbar_items = [toolbar items];
	
	int i = 0, len = [toolbar_items count];
	for( ; i < len; i++ )
	{
		[[toolbar_items objectAtIndex:i] setEnabled:vm_on];
	}
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar 
	 itemForItemIdentifier:(NSString *)itemIdentifier 
 willBeInsertedIntoToolbar:(BOOL)flag
{
	NSToolbarItem * toolbar_item;
	
	if( [itemIdentifier isEqual:@"add"] )
	{
        toolbar_item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
		[toolbar_item setLabel:@"Add Shred"];
		[toolbar_item setAction:@selector(add:)];
		[toolbar_item setImage:[NSImage imageNamed:@"add.tiff"]];
	}
	
	else if( [itemIdentifier isEqual:@"remove"] )
	{
#if defined( USE_POPUP_TOOLBAR_ITEMS ) && USE_POPUP_TOOLBAR_ITEMS
        KBPopUpToolbarItem * popup_tb_item = [[KBPopUpToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
        remove_menu = [[NSMenu new] autorelease];
        [remove_menu addItem:[NSMenuItem separatorItem]];
        NSMenuItem * mi = [[NSMenuItem alloc] initWithTitle:@"Remove All Child Shreds"
                                                     action:@selector(replaceShred:)
                                              keyEquivalent:@""];
        [mi autorelease];
        [mi setTag:0];
        [remove_menu addItem:mi];
        [popup_tb_item setMenu:remove_menu];
		toolbar_item = popup_tb_item;
		[toolbar_item setLabel:@"Remove Shred"];
		[toolbar_item setAction:@selector(remove:)];
        NSImage * remove_image = [NSImage imageNamed:@"remove.tiff"];
        [remove_image setScalesWhenResized:YES];
        // TODO: scale these in image editor, ditch crappy runtime scaling
        [remove_image setSize:NSMakeSize( 32, 32 )];
		[toolbar_item setImage:remove_image];
#else
        toolbar_item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
		[toolbar_item setLabel:@"Remove Shred"];
		[toolbar_item setAction:@selector(remove:)];
		[toolbar_item setImage:[NSImage imageNamed:@"remove.tiff"]];
#endif
	}
	
	else if( [itemIdentifier isEqual:@"replace"] )
	{
#if defined( USE_POPUP_TOOLBAR_ITEMS ) && USE_POPUP_TOOLBAR_ITEMS
        KBPopUpToolbarItem * popup_tb_item = [[KBPopUpToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
        replace_menu = [[NSMenu new] autorelease];
        [replace_menu addItem:[NSMenuItem separatorItem]];
        NSMenuItem * mi = [[NSMenuItem alloc] initWithTitle:@"Replace All Child Shreds"
                                                     action:@selector(replaceShred:)
                                              keyEquivalent:@""];
        [mi autorelease];
        [mi setTag:0];
        [replace_menu addItem:mi];
        [popup_tb_item setMenu:replace_menu];
		toolbar_item = popup_tb_item;
        [toolbar_item setLabel:@"Replace Shred"];
		[toolbar_item setAction:@selector(replace:)];
        NSImage * replace_image = [NSImage imageNamed:@"replace.tiff"];
        [replace_image setScalesWhenResized:YES];
        // TODO: scale these in image editor, ditch crappy runtime scaling
        [replace_image setSize:NSMakeSize( 32, 32 )];
		[toolbar_item setImage:replace_image];
#else
        toolbar_item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
		[toolbar_item setLabel:@"Replace Shred"];
		[toolbar_item setAction:@selector(replace:)];
		[toolbar_item setImage:[NSImage imageNamed:@"replace.tiff"]];
#endif
    }
	
	else if( [itemIdentifier isEqual:@"removeall"] )
	{
        toolbar_item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
		[toolbar_item setLabel:@"Remove All Shreds"];
		[toolbar_item setAction:@selector(removeall:)];
		[toolbar_item setImage:[NSImage imageNamed:@"removeall.tiff"]];
	}
	
	else if( [itemIdentifier isEqual:@"removelast"] )
	{
        toolbar_item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
		[toolbar_item setLabel:@"Remove Last Shred"];
		[toolbar_item setAction:@selector(removelast:)];
		[toolbar_item setImage:[NSImage imageNamed:@"removelast.tiff"]];
	}
	
    [toolbar_item autorelease];

	[toolbar_item setEnabled:vm_on];
	
	[toolbar_item setTag:1];
	[toolbar_item setTarget:self];
	
	return toolbar_item;
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar *)toolbar
{
	return [NSArray arrayWithObjects:@"add", @"remove", @"removelast", 
		@"removeall", NSToolbarFlexibleSpaceItemIdentifier,	@"replace", nil];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar *)toolbar
{
	return [NSArray arrayWithObjects:@"add", @"replace", @"remove", 
		NSToolbarFlexibleSpaceItemIdentifier, @"removelast", @"removeall", nil];
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)toolbar_item
{
	if( [toolbar_item tag] == 1 )
		return vm_on;
	
	else
		return YES;
}

@end
