/*----------------------------------------------------------------------------
miniAudicle
GUI to chuck audio programming environment

Copyright (c) 2005 Spencer Salazar.  All rights reserved.
http://chuck.cs.princeton.edu/
http://soundlab.cs.princeton.edu/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// file: mAPreferencesWindow.cpp
// desc: preferences window controller
//
// author: Spencer Salazar (ssalazar@cs.princeton.edu)
// date: Fall 2006
//-----------------------------------------------------------------------------

#include "chuck_def.h"

#include "wx/wx.h"
#include "wx/config.h"
#include "wx/colordlg.h"
#include "wx/fontdlg.h"

#include "mAPreferencesWindow.h"
#include "mAEvents.h"

const wxString mAPreferencesParentFrameWidth = _T( "/GUI/ParentFrame/width" );
const wxString mAPreferencesParentFrameHeight = _T( "/GUI/ParentFrame/height" );
const wxString mAPreferencesParentFrameX = _T( "/GUI/ParentFrame/x" );
const wxString mAPreferencesParentFrameY = _T( "/GUI/ParentFrame/y" );
const wxString mAPreferencesParentFrameMaximize = _T( "/GUI/ParentFrame/maximize" );
const wxString mAPreferencesVMMonitorWidth = _T( "/GUI/VMMonitor/width" );
const wxString mAPreferencesVMMonitorHeight = _T( "/GUI/VMMonitor/height" );
const wxString mAPreferencesVMMonitorX = _T( "/GUI/VMMonitor/x" );
const wxString mAPreferencesVMMonitorY = _T( "/GUI/VMMonitor/y" );
const wxString mAPreferencesVMMonitorMaximize = _T( "/GUI/VMMonitor/maximize" );
const wxString mAPreferencesConsoleMonitorWidth = _T( "/GUI/ConsoleMonitor/width" );
const wxString mAPreferencesConsoleMonitorHeight = _T( "/GUI/ConsoleMonitor/height" );
const wxString mAPreferencesConsoleMonitorX = _T( "/GUI/ConsoleMonitor/x" );
const wxString mAPreferencesConsoleMonitorY = _T( "/GUI/ConsoleMonitor/y" );
const wxString mAPreferencesConsoleMonitorMaximize = _T( "/GUI/ConsoleMonitor/maximize" );

const wxString mAPreferencesFontName = _T( "/GUI/Editing/FontName" );
const wxString mAPreferencesFontSize = _T( "/GUI/Editing/FontSize" );

const wxString mAPreferencesSyntaxColoringEnabled = _T( "/GUI/Editing/SyntaxColoringEnabled" );
const wxString mAPreferencesSyntaxColoringNormalText = _T( "/GUI/Editing/SyntaxColoring/NormalText" );
const wxString mAPreferencesSyntaxColoringKeywords = _T( "/GUI/Editing/SyntaxColoring/Keywords" );
const wxString mAPreferencesSyntaxColoringComments = _T( "/GUI/Editing/SyntaxColoring/Comments" );
const wxString mAPreferencesSyntaxColoringStrings = _T( "/GUI/Editing/SyntaxColoring/Strings" );
const wxString mAPreferencesSyntaxColoringNumbers = _T( "/GUI/Editing/SyntaxColoring/Numbers" );
const wxString mAPreferencesSyntaxColoringBackground = _T( "/GUI/Editing/SyntaxColoring/Background" );

const wxString mAPreferencesUseTabs = _T( "/GUI/Editing/UsesTabs" );
const wxString mAPreferencesTabSize = _T( "/GUI/Editing/TabSize" );
const wxString mAPreferencesShowLineNumbers = _T( "/GUI/Editing/ShowLineNumbers" );

const wxString mAPreferencesCurrentDirectory = _T( "/Miscellaneous/CurrentDirectory" );


struct __sc_token
{
	__sc_token( const wxString * _key, const wxString & _name, long _color )
		: key( _key ), name( _name ), color( _color )
	{ }

	const wxString * key;
	const wxString name;
	const long color;
};

__sc_token sc_tokens[] = 
{
	__sc_token( &mAPreferencesSyntaxColoringNormalText, _T( "Normal Text" ), 0x000000 ),
	__sc_token( &mAPreferencesSyntaxColoringKeywords,	_T( "Keywords" ),	 0x0000ff ),
	__sc_token( &mAPreferencesSyntaxColoringComments,	_T( "Comments" ),	 0x609010 ),
	__sc_token( &mAPreferencesSyntaxColoringStrings,	_T( "Strings" ),	 0x404040 ),
	__sc_token( &mAPreferencesSyntaxColoringNumbers,	_T( "Numbers" ),	 0xd48010 ),
	__sc_token( &mAPreferencesSyntaxColoringBackground,	_T( "Background" ),	 0xffffff ),
	__sc_token( NULL, _T( "" ), 0 )
};

mAPreferencesWindow::mAPreferencesWindow( mAWindowParent * parent, 
	wxWindowID id, const wxString & title, const wxPoint & pos, 
	const wxSize & size, long style )
	: mAWindowSuper( parent, id, title, pos, size, style )
{
	SetSize( 350, 350 );
	Center();
	Hide();

#ifdef __WINDOWS_DS__
	/* the default background is white, which looks bad */
	SetOwnBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_BTNFACE ) );
#endif
	
	int x, y, w, h;
	GetPosition( &x, &y );
	GetSize( &w, &h );

	// initialize top level sizer
	wxBoxSizer * top = new wxBoxSizer( wxVERTICAL );

	// create the notebook (tabbed-view) control
	notebook = new wxNotebook( this, wxID_ANY );
	top->Add( notebook, 2, wxALL | wxEXPAND, 10 );

	// create/populate the audio page of the notebook
	audio_page = new wxPanel( notebook, wxID_ANY );

	notebook->AddPage( audio_page, _T( "Audio" ) );
	
	wxBoxSizer * audio_page_top = new wxBoxSizer( wxVERTICAL );
	wxSizer * audio_page_sizers[4];

	audio_page_sizers[0] = new wxBoxSizer( wxHORIZONTAL );
	
	// enable real-time audio
	enable_audio = new wxCheckBox( audio_page, wxID_ANY, 
								   _T( "Enable audio" ) );
	enable_audio->SetValue( true );
	audio_page_sizers[0]->Add( enable_audio );

	audio_page_sizers[0]->Add( 10, 0, 1 );

	// enable network thread
	enable_network = new wxCheckBox( audio_page, wxID_ANY, 
									 _T( "Accept network VM commands" ) );
	audio_page_sizers[0]->Add( enable_network );

	audio_page_top->Add( audio_page_sizers[0], 0, 
						 wxTOP | wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );

	// 
	audio_page_sizers[1] = new wxFlexGridSizer( 2, 2, 5, 10 );
	( ( wxFlexGridSizer * ) audio_page_sizers[1] )->AddGrowableCol( 1 );
	
	// audio output selector
	audio_page_sizers[1]->Add( new wxStaticText( audio_page, wxID_ANY, 
											     _T( "Audio Output" ) ),
							   0, wxALIGN_CENTER_VERTICAL );
	audio_output = new wxChoice( audio_page, wxID_ANY ); // TODO: needs a real id
	audio_output->Append( _T( "Default" ) );
	audio_output->Append( _T( "(nothing)" ) );
	audio_output->SetSelection( 0 );
	audio_page_sizers[1]->Add( audio_output, 0, wxEXPAND | wxALIGN_CENTER_VERTICAL );

	// audio input selector
	audio_page_sizers[1]->Add( new wxStaticText( audio_page, wxID_ANY, 
											     _T( "Audio Input" ) ),
							   0, wxALIGN_CENTER_VERTICAL );
	audio_input = new wxChoice( audio_page, wxID_ANY ); // TODO: needs a real id
	audio_input->Append( _T( "Default" ) );
	audio_input->Append( _T( "(nothing)" ) );
	audio_input->SetSelection( 0 );
	audio_page_sizers[1]->Add( audio_input, 0, wxEXPAND | wxALIGN_CENTER_VERTICAL );

    //
	audio_page_top->Add( audio_page_sizers[1], 0, 
						 wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );

	//
	audio_page_sizers[2] = new wxBoxSizer( wxHORIZONTAL );

	// output channels
	audio_page_sizers[2]->Add( new wxStaticText( audio_page, wxID_ANY, 
												 _T( "Output Channels:" ) ),
							   0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5 );
	output_channels = new wxChoice( audio_page, wxID_ANY );
	output_channels->Append( _T( "1" ) );
	output_channels->Append( _T( "2" ) );
	output_channels->Append( _T( "32" ) );
	output_channels->SetSelection( 1 );
	audio_page_sizers[2]->Add( output_channels, 0, wxALIGN_CENTER_VERTICAL );

	// spacer
	audio_page_sizers[2]->AddStretchSpacer();

	// input channels
	audio_page_sizers[2]->Add( new wxStaticText( audio_page, wxID_ANY, 
												 _T( "Input Channels:" ) ),
							   0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5 );
	input_channels = new wxChoice( audio_page, wxID_ANY );
	input_channels->Append( _T( "1" ) );
    input_channels->Append( _T( "2" ) );
	input_channels->Append( _T( "32" ) );
	input_channels->SetSelection( 1 );
	audio_page_sizers[2]->Add( input_channels, 0, wxALIGN_CENTER_VERTICAL );

    //
	audio_page_top->Add( audio_page_sizers[2], 0,
						 wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );

    //
    audio_page_sizers[3] = new wxBoxSizer( wxHORIZONTAL );

    // sample rate
    audio_page_sizers[3]->Add( new wxStaticText( audio_page, wxID_ANY,
                                                 _T( "Sample Rate:" ) ),
                               0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5 );
    sample_rate = new wxChoice( audio_page, wxID_ANY );
    sample_rate->Append( _T( "11025" ) );
    sample_rate->Append( _T( "22050" ) );
    sample_rate->Append( _T( "44100" ) );
    sample_rate->Append( _T( "192000" ) );
    sample_rate->SetSelection( 2 );
    audio_page_sizers[3]->Add( sample_rate, 0, wxALIGN_CENTER_VERTICAL );

    // spacer
    audio_page_sizers[3]->AddStretchSpacer();

    // buffer size
    audio_page_sizers[3]->Add( new wxStaticText( audio_page, wxID_ANY,
                                                 _T( "Buffer Size:" ) ),
                               0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5 );
    sample_rate = new wxChoice( audio_page, wxID_ANY );
    sample_rate->Append( _T( "128" ) );
    sample_rate->Append( _T( "256" ) );
    sample_rate->Append( _T( "512" ) );
    sample_rate->Append( _T( "1024" ) );
    sample_rate->SetSelection( 2 );
    audio_page_sizers[3]->Add( sample_rate, 0, wxALIGN_CENTER_VERTICAL );

    //
    audio_page_top->Add( audio_page_sizers[3], 0,
						 wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );
    
	audio_page_top->AddSpacer( 10 );
	
	// add audio probe 
	audio_page_top->Add( new wxButton( audio_page, 
									   mAID_PREFS_PROBE_AUDIO, 
									   _T( "Probe Audio Interfaces" ) ),
						 0, wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10 );

	audio_page_top->AddSpacer( 10 );
	
#ifdef __WINDOWS_DS__
	wxStaticText * restart = new wxStaticText( audio_page, wxID_ANY,
											   _T( "Changes will not take effect until miniAudicle is restarted." ) );
#else
	wxStaticText * restart = new wxStaticText( audio_page, wxID_ANY,
											   _T( "Changes will not take effect until virtual machine is restarted." ) );
#endif
	restart->GetFont().SetPointSize( 10 );
	audio_page_top->Add( restart, 0, 
						 wxALIGN_CENTER | wxLEFT | wxRIGHT | wxBOTTOM, 10 );

	audio_page_top->AddStretchSpacer();
										   
	//
	audio_page->SetSizer( audio_page_top );
	
	// create/populate the editing page of the notebook
	editing_page = new wxPanel( notebook );
	notebook->AddPage( editing_page, _T( "Editing" ) );

	wxBoxSizer * editing_page_top = new wxBoxSizer( wxVERTICAL );
	wxSizer * editing_page_sizers[3];

	// add font selection
	editing_page_sizers[0] = new wxBoxSizer( wxHORIZONTAL );

	editing_page_sizers[0]->Add( new wxStaticText( editing_page, wxID_ANY,
												   _T( "Editing font:" ) ),
								 0, wxALIGN_CENTER_VERTICAL );
	editing_page_sizers[0]->AddStretchSpacer();
	editing_page_sizers[0]->Add( new wxButton( editing_page, mAID_PREFS_CHOOSE_FONT,
											   _T( "Set Editing Font..." ) ) );
	Connect( mAID_PREFS_CHOOSE_FONT, wxEVT_COMMAND_BUTTON_CLICKED,
		wxCommandEventHandler( mAPreferencesWindow::OnChooseFont ) );

	//
    editing_page_top->Add( editing_page_sizers[0], 0,
						   wxTOP | wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );

	// 
	font_display = new wxTextCtrl( editing_page, wxID_ANY, _T( "Courier - 10pt" ),
								   wxDefaultPosition, wxDefaultSize, wxTE_READONLY );
	editing_page_top->Add( font_display, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );
	
	// syntax coloring
	enable_coloring = new wxCheckBox( editing_page, wxID_ANY, _T( "Enable syntax coloring" ) );
	enable_coloring->SetValue( true );

	editing_page_top->Add( enable_coloring, 0, wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );
	
	//
	editing_page_sizers[1] = new wxBoxSizer( wxHORIZONTAL );
	
	editing_page_sizers[1]->AddSpacer( 10 );

	//
	color_items = new wxChoice( editing_page, mAID_PREFS_SYNTAX_COLORING_ITEM_CHANGED );
	for( int i = 0; sc_tokens[i].key != NULL; i++ )
		color_items->Append( sc_tokens[i].name );
	color_items->SetSelection( 0 );
	editing_page_sizers[1]->Add( color_items );
    Connect( mAID_PREFS_SYNTAX_COLORING_ITEM_CHANGED, wxEVT_COMMAND_CHOICE_SELECTED,
        wxCommandEventHandler( mAPreferencesWindow::OnSelectSyntaxColoringItem ) );

	//
	editing_page_sizers[1]->AddSpacer( 10 );

	//
	color_button = new wxButton( editing_page, mAID_PREFS_CHOOSE_SYNTAX_COLOR,
								 _T( "" ), wxDefaultPosition,
								 wxSize( -1, color_items->GetSize().GetHeight() ) );
	color_button->SetBackgroundColour( *wxBLACK );
	editing_page_sizers[1]->Add( color_button );
    Connect( mAID_PREFS_CHOOSE_SYNTAX_COLOR, wxEVT_COMMAND_BUTTON_CLICKED,
             wxCommandEventHandler( mAPreferencesWindow::OnChooseSyntaxColor ) );

	//
	editing_page_top->Add( editing_page_sizers[1], 0, 
						   wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );
	editing_page_top->AddSpacer( 5 );

	// tabs
	tab_key_tabs = new wxCheckBox( editing_page, wxID_ANY,
								   _T( "Tab key inserts tabs, not spaces" ) );
	tab_key_tabs->SetValue( true );

	editing_page_top->Add( tab_key_tabs, 0, 
						   wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );

	// 
	editing_page_sizers[2] = new wxBoxSizer( wxHORIZONTAL );
	editing_page_sizers[2]->AddSpacer( 10 );
	editing_page_sizers[2]->Add( new wxStaticText( editing_page, wxID_ANY, 
												   _T( "Tab width:" ) ),
								 0, wxALIGN_CENTER_VERTICAL | wxRIGHT, 5 );

	//
	tab_size = new wxTextCtrl( editing_page, wxID_ANY, _T( "4" ) );
	tab_size->SetMaxLength( 2 );
	wxSize _size = tab_size->ConvertDialogToPixels( wxSize( 16, 10 ) );
	tab_size->SetSize( _size.GetWidth(), -1 );
	tab_size->SetMaxSize( tab_size->GetSize() );
	editing_page_sizers[2]->Add( tab_size, 0, wxSHAPED );

	// 
	editing_page_top->Add( editing_page_sizers[2], 0, 
						   wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );
	editing_page_top->AddSpacer( 5 );

	// line numbers
	line_numbers = new wxCheckBox( editing_page, wxID_ANY, 
								   _T( "Display line numbers" ) );
	line_numbers->SetValue( true );
	
	editing_page_top->Add( line_numbers, 0, 
						   wxLEFT | wxRIGHT | wxBOTTOM | wxEXPAND, 10 );

	//
	editing_page_top->AddStretchSpacer();
	editing_page->SetSizer( editing_page_top );

	// create/populate the miscellaneous page of the notebook
	misc_page = new wxPanel( notebook );
	notebook->AddPage( misc_page, _T( "Miscellaneous" ) );

	wxBoxSizer * misc_page_top = new wxBoxSizer( wxVERTICAL );
	wxSizer * misc_page_sizers[2];

	// current directory
	misc_page_sizers[0] = new wxBoxSizer( wxHORIZONTAL );
	misc_page_sizers[0]->Add( new wxStaticText( misc_page, wxID_ANY,
												_T( "Current Directory:" ) ),
							  0, wxALIGN_CENTER_VERTICAL );

	misc_page_sizers[0]->AddStretchSpacer();
	
	misc_page_sizers[0]->Add( new wxButton( misc_page, mAID_PREFS_CHOOSE_CWD,
											_T( "Select..." ) ) );
    Connect( mAID_PREFS_CHOOSE_CWD, wxEVT_COMMAND_BUTTON_CLICKED,
             wxCommandEventHandler( mAPreferencesWindow::OnChooseCurrentDirectory ) );

	//
	misc_page_top->Add( misc_page_sizers[0], 0, 
						wxTOP | wxRIGHT | wxLEFT | wxBOTTOM | wxEXPAND, 10 );

	cwd_display = new wxTextCtrl( misc_page, wxID_ANY, wxGetHomeDir(),
								  wxDefaultPosition, wxDefaultSize, wxTE_READONLY );
	misc_page_top->Add( cwd_display, 0, 
						wxRIGHT | wxLEFT | wxBOTTOM | wxEXPAND, 10 );

	misc_page_top->Add( new wxStaticText( misc_page, wxID_ANY, 
										  _T( "Relative filenames in ChucK programs (used in SndBuf, WvIn, and" ) ),
						0, wxRIGHT | wxLEFT, 10 );
	misc_page_top->Add( new wxStaticText( misc_page, wxID_ANY, 
										  _T( "WvOut, for example) will be interpreted relative to this directory." ) ),
						0, wxRIGHT | wxLEFT | wxBOTTOM, 10 );

	//
	misc_page_top->AddStretchSpacer();

	//
	misc_page->SetSizer( misc_page_top );
	
	// arrange the "restore defaults"/"OK"/"Cancel" buttons
	wxBoxSizer * ok_cancel = new wxBoxSizer( wxHORIZONTAL );

	ok_cancel->Add( new wxButton( this, wxID_DEFAULT, 
								  _T( "Restore Defaults" ) ) );

	ok_cancel->AddStretchSpacer();

	ok = new wxButton( this, wxID_OK, _T( "OK" ) );
	ok_cancel->Add( ok, 0, wxLEFT | wxRIGHT, 10 );

	ok_cancel->Add( new wxButton( this, wxID_CANCEL, _T( "Cancel" ) ) );

	top->Add( ok_cancel, 0, wxEXPAND | wxBOTTOM | wxRIGHT | wxLEFT, 10 );

	// set window sizer
	SetSizer( top );

	// set the window size to the minimum possible
	top->Fit( this );

    // set minimum sizes for choice boxes
    audio_page_sizers[2]->SetItemMinSize( output_channels, 
                                          output_channels->GetSize().GetWidth(),
                                          output_channels->GetSize().GetHeight() );
    output_channels->Delete( 2 );

    audio_page_sizers[2]->SetItemMinSize( input_channels, 
                                          input_channels->GetSize().GetWidth(),
                                          input_channels->GetSize().GetHeight() );
    input_channels->Delete( 2 );

	Connect( wxID_ANY, wxEVT_CLOSE_WINDOW, 
		wxCloseEventHandler( mAPreferencesWindow::OnClose ) );
	Connect( wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED,
		wxCommandEventHandler( mAPreferencesWindow::OnOk ) );
	Connect( wxID_CANCEL, wxEVT_COMMAND_BUTTON_CLICKED,
		wxCommandEventHandler( mAPreferencesWindow::OnCancel ) );

	LoadPreferencesToGUI();
}

mAPreferencesWindow::~mAPreferencesWindow()
{
	
}

void mAPreferencesWindow::OnPreferencesCommand( wxCommandEvent & event )
{
	Show();
	Raise();
	ok->SetFocus();
}

void mAPreferencesWindow::OnClose( wxCloseEvent & event )
{
	if( event.CanVeto() )
	{
		event.Veto();
		Hide();
	}
	else
		Destroy();
}

void mAPreferencesWindow::OnOk( wxCommandEvent & event )
{
	Hide();
}

void mAPreferencesWindow::OnCancel( wxCommandEvent & event )
{
	Hide();
    LoadPreferencesToGUI();
}

void mAPreferencesWindow::OnChooseFont( wxCommandEvent & event )
{
	wxFontDialog font_dialog( this, font_dialog_data );
    if( font_dialog.ShowModal() == wxID_OK )
    {
        font_dialog_data = font_dialog.GetFontData();
	    wxString t_string;
        wxFont chosen_font = font_dialog_data.GetChosenFont();
        wxString face_name = chosen_font.GetFaceName();
        int point_size = chosen_font.GetPointSize();
	    t_string.Append( face_name ).Append( _T( " - " ) ).Append( wxString::Format( _T( "%i" ), point_size) ).Append( _T( " pt" ) );
	    font_display->SetValue( t_string );
    }
}

void mAPreferencesWindow::OnSelectSyntaxColoringItem( wxCommandEvent & event )
{
	long color = colors[color_items->GetStringSelection()];
	color_button->SetBackgroundColour( wxColor( ( color >> 16 ) & 0xff, 
                                                ( color >> 8 ) & 0xff, 
                                                ( color ) & 0xff  ) );

}

void mAPreferencesWindow::OnChooseSyntaxColor( wxCommandEvent & event )
{
    wxColour new_color = wxGetColourFromUser( this, color_button->GetBackgroundColour() );

    if( new_color.Ok() )
    {
        color_button->SetBackgroundColour( new_color );
        long color = ( new_color.Red() << 16 ) | 
                     ( new_color.Green() << 8 ) | 
                     ( new_color.Blue() );
        colors[color_items->GetStringSelection()] = color;
    }
}

void mAPreferencesWindow::OnChooseCurrentDirectory( wxCommandEvent & event )
{
    wxString new_cwd = wxDirSelector( _T( "Choose Current Directory" ), 
                                      cwd_display->GetValue() );
    if( !new_cwd.empty() )
        cwd_display->SetValue( new_cwd );
}

void mAPreferencesWindow::LoadPreferencesToGUI()
// assumes command line preferences have already been parsed
{
	wxConfigBase * config = wxConfigBase::Get();

	wxString str;
	long l;
    bool b;
	
	str = wxGetHomeDir();
	config->Read( _T( "/Miscellaneous/CurrentDirectory" ), &str );
	cwd_display->SetValue( str );

	config->Read( _T( "/GUI/Editing/FontName" ), &str, 
		          wxFont( 10, wxFONTFAMILY_MODERN, wxNORMAL, 
                          wxFONTWEIGHT_NORMAL ).GetFaceName() );
	config->Read( _T( "/GUI/Editing/FontSize" ), &l, 11 );
	wxString t_string;
	t_string.Append( str ).Append( _T( " - " ) ).Append( wxString::Format( _T( "%li" ), l ) ).Append( _T( " pt" ) );
	font_display->SetValue( t_string );

    font_dialog_data.SetInitialFont( wxFont( l, wxFONTFAMILY_MODERN, wxNORMAL,
                                     wxFONTWEIGHT_NORMAL, false, str ) );
	font_dialog_data.EnableEffects( false );
	font_dialog_data.SetAllowSymbols( false );

	config->Read( mAPreferencesSyntaxColoringEnabled, &b, true );
	enable_coloring->SetValue( b );

	for( int i = 0; sc_tokens[i].key != NULL; i++ )
	{
		config->Read( *sc_tokens[i].key, &l, sc_tokens[i].color );
		colors[sc_tokens[i].name] = l;
	}

	long color = colors[color_items->GetStringSelection()];
	color_button->SetBackgroundColour( wxColor( ( color >> 16 ) & 0xff, 
                                                ( color >> 8 ) & 0xff, 
                                                ( color ) & 0xff  ) );

	config->Read( mAPreferencesUseTabs, &b, true );
	tab_key_tabs->SetValue( b );

	config->Read( mAPreferencesTabSize, &l, 4 );
    tab_size->SetValue( wxString::Format( _T( "%i" ), l ) );

	config->Read( mAPreferencesShowLineNumbers, &b, true );
	line_numbers->SetValue( b );

}


