/*----------------------------------------------------------------------------
miniAudicle
GUI to chuck audio programming environment

Copyright (c) 2005 Spencer Salazar.  All rights reserved.
http://chuck.cs.princeton.edu/
http://soundlab.cs.princeton.edu/

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
U.S.A.
-----------------------------------------------------------------------------*/

//-----------------------------------------------------------------------------
// file: mAUIElements.cpp
// desc: for supporting UI controls created and used in ChucK code
//       wxWidgets specific code
//
// author: Spencer Salazar (ssalazar@princeton.edu)
// date: Summer 2006
//-----------------------------------------------------------------------------

#include "mAUIElements.h"
#include "mAParentFrame.h"
#include "miniAudicle_ui_elements.h"

#include "wx/wx.h"
#include "wx/event.h"
#include "wx/window.h"

using namespace std;

//namespace miniAudicle
//{
namespace UI
{

DECLARE_EVENT_TYPE( mAEVT_UI_INIT, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_INIT )
DECLARE_EVENT_TYPE( mAEVT_UI_DISPLAY, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_DISPLAY )
DECLARE_EVENT_TYPE( mAEVT_UI_HIDE, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_HIDE )
DECLARE_EVENT_TYPE( mAEVT_UI_DESTROY, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_DESTROY )
DECLARE_EVENT_TYPE( mAEVT_UI_SIZE, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_SIZE )
DECLARE_EVENT_TYPE( mAEVT_UI_POSITION, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_POSITION )
DECLARE_EVENT_TYPE( mAEVT_UI_NAME, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_NAME )
DECLARE_EVENT_TYPE( mAEVT_UI_SETPARENT, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_SETPARENT )

class mAElement : public wxEvtHandler
{
public:
    mAElement() : wxEvtHandler()
    {
        master_view = NULL;
        owner = NULL;

        Connect( mAEVT_UI_INIT, wxCommandEventHandler( mAElement::Init ) );
        Connect( mAEVT_UI_DISPLAY, wxCommandEventHandler( mAElement::Display ) );
        Connect( mAEVT_UI_HIDE, wxCommandEventHandler( mAElement::Hide ) );
        Connect( mAEVT_UI_DESTROY, wxCommandEventHandler( mAElement::Destroy ) );
        Connect( mAEVT_UI_SIZE, wxCommandEventHandler( mAElement::SetSize ) );
        Connect( mAEVT_UI_POSITION, wxCommandEventHandler( mAElement::SetPosition ) );
        Connect( mAEVT_UI_NAME, wxCommandEventHandler( mAElement::SetName ) );
        Connect( mAEVT_UI_SETPARENT, wxCommandEventHandler( mAElement::SetParent ) );
    }

    virtual void Link( Element * e )
    {
        owner = e;
    }

    virtual void Init( wxCommandEvent & event )
    {

    }

    virtual void Display( wxCommandEvent & event )
    {
        if( master_view )
        {
            master_view->Show();
            if( master_view->IsTopLevel() )
                master_view->Raise();
        }
    }

    virtual void Hide( wxCommandEvent & event )
    {
        if( master_view )
            master_view->Hide();
    }

    virtual void Destroy( wxCommandEvent & event )
    {
        if( master_view )
        {
            master_view->Hide();
            master_view->Destroy();
            master_view = NULL;
        }

//        delete this;
// TODO: set up an event that this function can post so that the mAApp will delete this
    }

    virtual void SetSize( wxCommandEvent & event )
    {
        if( master_view )
            master_view->SetClientSize( owner->get_width(), owner->get_height() );
    }

    virtual void SetPosition( wxCommandEvent & event )
    {
        if( master_view )
        {
            int x = owner->get_x(), y = owner->get_y();
            master_view->Move( x, y );
            if( x == -1 )
            {
                if( y == -1 )
                    master_view->Center( wxHORIZONTAL | wxVERTICAL );
                else
                    master_view->Center( wxHORIZONTAL );
            }
            else if( y == -1 )
                master_view->Center( wxVERTICAL );
        }
    }

    virtual void SetName( wxCommandEvent & event )
    {
        if( master_view )
            master_view->SetTitle( wxString( owner->get_name().c_str(), wxConvUTF8 ) );
    }

    virtual void SetParent( wxCommandEvent & event ) = 0;
    virtual void RemoveParent( wxCommandEvent & event ) = 0;

protected:
    Element * owner;
    wxWindow * master_view;
};

class mAView : public mAElement
{
public:
    mAView() : mAElement()
    {
        frame = NULL;
    }

    virtual void Init( wxCommandEvent & event )
    {
        frame = wxGetApp().CreateChildFrame();
        master_view = frame;
#ifdef __WINDOWS_DS__
	    /* the default background is white, which looks bad */
	    frame->SetOwnBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_BTNFACE ) );
#endif
        frame->Hide();

        SetSize( event );
        SetPosition( event );
        SetName( event );

        frame->PushEventHandler( this );
        Connect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( mAView::OnCloseFrame ) );
    }

    virtual void Display( wxCommandEvent & event )
    {
        if( frame )
        {
            frame->Show();
            frame->Raise();
        }
    }

    virtual void Destroy( wxCommandEvent & event )
    {
        wxGetApp().RemoveMenuBar( ( mAMenuBar * ) frame->GetMenuBar() );
        frame->Destroy();
//        delete this;
// TODO: set up an event that this function can post so that the mAApp will delete this
    }

    virtual mAFrameType * GetFrame()
    {
        return frame;
    }

    virtual void SetParent( wxCommandEvent & event ){ }
    virtual void RemoveParent( wxCommandEvent & event ){ }

    virtual void OnCloseFrame( wxCloseEvent & event )
    {
        if( event.CanVeto() )
        {
            event.Veto();
            frame->Hide();
        }

        else
        {
            wxGetApp().RemoveMenuBar( ( mAMenuBar * ) frame->GetMenuBar() );
            frame->Destroy();
        }
    }

protected:
    mAFrameType * frame;
};

struct NativeView
{
    mAView * view;
};

t_CKBOOL View::init()
{
    native_data = new NativeView;
    native_data->view = new mAView();
    native_data->view->Link( this );

    // defaults
    x = -1; // center
    y = -1; // center
    w = 200;
    h = 200;
    name = "";

    wxCommandEvent event( mAEVT_UI_INIT );
    native_data->view->AddPendingEvent( event );
    
    return TRUE;
}

t_CKBOOL View::display()
{
    wxCommandEvent event( mAEVT_UI_DISPLAY );
    native_data->view->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL View::hide()
{    
    wxCommandEvent event( mAEVT_UI_HIDE );
    native_data->view->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL View::destroy()
{    
    wxCommandEvent event( mAEVT_UI_DESTROY );
    native_data->view->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL View::set_name( std::string & name )
{
    Element::set_name( name );
    
    wxCommandEvent event( mAEVT_UI_NAME );
    native_data->view->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL View::set_parent( View * p )
{
    return TRUE;
}

t_CKBOOL View::set_size( t_CKFLOAT w, t_CKFLOAT h )
{
    Element::set_size( w, h );
    
    wxCommandEvent event( mAEVT_UI_SIZE );
    native_data->view->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL View::set_position( t_CKFLOAT x, t_CKFLOAT y )
{
    Element::set_position( x, y );
    
    wxCommandEvent event( mAEVT_UI_POSITION );
    native_data->view->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL View::add_element( Element * e )
{
    if( e == NULL )
        return FALSE;
    
    elements.push_back( e );
    e->set_parent( this );
    
    return TRUE;
}

t_CKBOOL View::remove_element( Element * e )
{
    if( e == NULL )
        return FALSE;
    
    std::vector< Element * >::size_type i, len = elements.size();
    
    for( i = 0; i < len; i++ )
    {
        if( elements[i] == e )
        {
            elements[i]->remove_parent( this );
            elements[i] = NULL;
        }
    }
    
    return TRUE;
}

/* in wxWidgets, sliders can only take integral values. to simulate a 
   continuous slider, we set the range to be higher than the possible 
   width of the slider, and then scale it to the user's requested range.  */
#define mASLIDER_MAX 32767 // slider minimum is 0

DECLARE_EVENT_TYPE( mAEVT_UI_RANGE, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_RANGE )
DECLARE_EVENT_TYPE( mAEVT_UI_VALUE, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_VALUE )
DECLARE_EVENT_TYPE( mAEVT_UI_UPDATE, -1 )
DEFINE_EVENT_TYPE( mAEVT_UI_UPDATE )

class mASlider : public mAElement
{
public:
    mASlider() : mAElement()
    {
        slider = NULL;
        s_owner = NULL;
        label = NULL;
        value = NULL;

        Connect( mAEVT_UI_RANGE, wxCommandEventHandler( mASlider::SetRange ) );
        Connect( mAEVT_UI_VALUE, wxCommandEventHandler( mASlider::SetValue ) );
    }

    virtual void Link( Slider * s )
    {
        owner = s_owner = s;
    }

    virtual void SetSize( wxCommandEvent & event )
    {
        if( master_view && slider && label && value )
        {
            master_view->SetSize( owner->get_width(), owner->get_height() );
            slider->Move( owner->get_left_margin(), owner->get_top_margin() + 
                Slider::default_text_height + Slider::default_inner_margin );
            slider->SetSize( owner->get_width() - owner->get_left_margin() - owner->get_right_margin(),
                             owner->get_height() - owner->get_top_margin() - owner->get_bottom_margin() -
                             Slider::default_inner_margin - Slider::default_text_height );
            label->Move( owner->get_left_margin(), owner->get_top_margin() );
            label->SetSize( owner->get_width() - owner->get_left_margin() - owner->get_right_margin(),
                            owner->get_height() - owner->get_top_margin() - owner->get_bottom_margin() -
                            Slider::default_inner_margin - Slider::default_slider_height );
            value->SetBestFittingSize( wxSize( -1 , 
                owner->get_height() - owner->get_top_margin() - owner->get_bottom_margin() -
                Slider::default_inner_margin - Slider::default_slider_height ) );
            value->Move( owner->get_width() - owner->get_right_margin() - value->GetClientSize().GetWidth(), 
                         owner->get_top_margin() );
        }
    }

    virtual void SetName( wxCommandEvent & event )
    {
        if( label )
            label->SetLabel( wxString( owner->get_name().c_str(), wxConvUTF8 ) );
    }

    virtual void SetParent( wxCommandEvent & event )
    {
        wxWindow * parent = ( ( mAView * )event.GetClientData() )->GetFrame();
        if( parent == NULL )
        /* when a default parent is used, View::init and Slider::set_parent are 
           both called in quick succession.  For some reason, wxWidgets does 
           not seem to guarantee that the resulting mAEVT_UI_INIT and SETPARENT
           are dispatched in the same order that they are posted.  Thus, we have
           to check the value of mAView::GetFrame, and if it is NULL, 
           continually repost the mAEVT_UI_SETPARENT until it is not NULL.  */
        {
            AddPendingEvent( event );
            return;
        }
        master_view = new wxWindow( parent, wxID_ANY );
        slider = new wxSlider( master_view, wxID_ANY, 0, 0, mASLIDER_MAX );
        label = new wxStaticText( master_view, wxID_ANY, _T( " " ), 
            wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE | wxALIGN_LEFT );
        value = new wxStaticText( master_view, wxID_ANY, _T( "0" ), 
            wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT );

        SetSize( event );
        SetPosition( event );
        SetName( event );
        SetRange( event );
        SetValue( event );

        slider->PushEventHandler( this );
        Connect( wxEVT_SCROLL_THUMBTRACK, wxScrollEventHandler( mASlider::SliderChanged ) );
#if ( wxABI_VERSION >= 20601 ) || !defined( wxABI_VERSION )
        Connect( wxEVT_SCROLL_CHANGED, wxScrollEventHandler( mASlider::SliderChanged ) );
#endif
    }

    virtual void RemoveParent( wxCommandEvent & event )
    {
        
    }

    virtual void SetRange( wxCommandEvent & event )
    {
        min = s_owner->get_min();
        max = s_owner->get_max();
    }

    virtual void SetValue( wxCommandEvent & event )
    {
        if( slider && value )
        {
            slider->SetValue( ( int ) ( mASLIDER_MAX * ( s_owner->get_value() - min ) / ( max - min ) ) );
            Update( event );
        }
    }

    virtual void Update( wxCommandEvent & event )
    {
        if( value )
        {
            value->SetLabel( wxString::Format( _T( "%f" ), s_owner->get_value() ) );
            SetSize( event );
        }
    }

    virtual void SliderChanged( wxScrollEvent & event )
    {
        t_CKFLOAT scaled_value = ( ( ( t_CKFLOAT ) slider->GetValue() ) / mASLIDER_MAX ) * ( max - min ) + min;
        
        s_owner->slider_changed( scaled_value );
        
        Update( event );
    }

protected:
    wxSlider * slider;
    wxStaticText * label;
    wxStaticText * value;
    Slider * s_owner;

    t_CKFLOAT min;
    t_CKFLOAT max;
};

struct NativeSlider
{
    mASlider * slider;
};

const t_CKUINT Slider::default_margin = 18;
const t_CKUINT Slider::default_width = 202;
const t_CKUINT Slider::default_slider_height = 21;
const t_CKUINT Slider::default_inner_margin = 5;
const t_CKUINT Slider::default_text_height = 17;

t_CKBOOL Slider::init()
{
    native_data = new NativeSlider;
    native_data->slider = new mASlider;
    native_data->slider->Link( this );

    // defaults
    x = -1; // center
    y = -1; // center
    w = default_width + default_margin * 2;
    h = default_slider_height + default_inner_margin + 
        default_text_height + default_margin * 2;
    lmargin = default_margin;
    rmargin = default_margin;
    tmargin = default_margin - 2;
    bmargin = default_margin - 2;
    name = "";
    e = NULL;

    value = 0;
    min = 0;
    max = 1;

    wxCommandEvent event( mAEVT_UI_INIT );
    native_data->slider->AddPendingEvent( event );
    
    return TRUE;
}

t_CKBOOL Slider::display()
{
    Element::display();
    
    wxCommandEvent event( mAEVT_UI_DISPLAY );
    native_data->slider->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL Slider::hide()
{
    Element::hide();
    
    wxCommandEvent event( mAEVT_UI_HIDE);
    native_data->slider->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL Slider::destroy()
{
    Element::destroy();
    
    wxCommandEvent event( mAEVT_UI_DESTROY );
    native_data->slider->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL Slider::set_name( string & name )
{    
    Element::set_name( name );
    
    wxCommandEvent event( mAEVT_UI_NAME );
    native_data->slider->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL Slider::set_parent( View * p )
{
    Element::set_parent( p );

    wxCommandEvent event( mAEVT_UI_SETPARENT );
    event.SetClientData( p->native_data->view );
    native_data->slider->AddPendingEvent( event );
    
    return TRUE;
}

t_CKBOOL Slider::remove_parent( View * p )
{
    Element::remove_parent( p );
    
    return TRUE;
}

t_CKBOOL Slider::set_size( t_CKFLOAT w, t_CKFLOAT h )
{
    Element::set_size( w, h );

    wxCommandEvent event( mAEVT_UI_SIZE );
    native_data->slider->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL Slider::set_position( t_CKFLOAT x, t_CKFLOAT y )
{
    Element::set_position( x, y );
    
    wxCommandEvent event( mAEVT_UI_POSITION );
    native_data->slider->AddPendingEvent( event );

    return TRUE;
}

t_CKDOUBLE Slider::get_value()
{
    return value;
}

t_CKBOOL Slider::set_value( t_CKDOUBLE v )
{
    if( v >= max )
        value = max;
    else if( v <= min )
        value = min;
    else
        value = v;

    wxCommandEvent event( mAEVT_UI_VALUE );
    native_data->slider->AddPendingEvent( event );

    return TRUE;
}

t_CKBOOL Slider::set_range( t_CKDOUBLE low, t_CKDOUBLE high )
{
    min = low;
    max = high;

    wxCommandEvent event( mAEVT_UI_RANGE );
    native_data->slider->AddPendingEvent( event );

    set_value( get_value() );

    return TRUE;
}

t_CKDOUBLE Slider::get_max()
{
    return max;
}

t_CKDOUBLE Slider::get_min()
{
    return min;
}

t_CKBOOL Slider::set_precision( t_CKUINT precision )
{
    this->precision = precision;
    
    // refresh
    set_value( get_value() );
    
    return TRUE;
}

t_CKUINT Slider::get_precision()
{
    return precision;
}

t_CKBOOL Slider::set_display_format( Slider::display_format f )
{
    this->df = f;
    
    // refresh
    set_value( get_value() );
    
    return TRUE;
}

Slider::display_format Slider::get_display_format()
{
    return df;
}

void Slider::slider_changed( t_CKFLOAT v )
{
    value = v;

    if( e != NULL )
        e->queue_broadcast();
}

t_CKBOOL Button::init()
{    
    return TRUE;
}

t_CKBOOL Button::display()
{
    Element::display();
    
    return TRUE;
}

t_CKBOOL Button::hide()
{
    Element::hide();
    
    return TRUE;
}

t_CKBOOL Button::destroy()
{
    Element::destroy();
    
    return TRUE;
}

t_CKBOOL Button::set_name( std::string & name )
{
    Element::set_name( name );
    
    return TRUE;
}

t_CKBOOL Button::set_size( t_CKFLOAT nw, t_CKFLOAT nh )
{
    Element::set_size( nw, nh );
        
    return TRUE;
}

t_CKBOOL Button::set_position( t_CKFLOAT x, t_CKFLOAT y )
{
    Element::set_position( x, y );
    
    return TRUE;
}

t_CKBOOL Button::set_parent( View * p )
{
    Element::set_parent( p );
    
    return TRUE;
}

t_CKBOOL Button::remove_parent( View * p )
{
    Element::remove_parent( p );
    
    return TRUE;
}

t_CKBOOL Button::get_state()
{
    return pushed;
}

t_CKBOOL Button::set_state( t_CKBOOL state )
{
    return FALSE;
}

t_CKBOOL Button::set_action_type( action_type t )
{    
    return FALSE;
}

Button::action_type Button::get_action_type()
{    
    return push_type;
}

void Button::button_changed()
{
    if( e != NULL )
        e->queue_broadcast();
    
    pushed = !pushed;
}

t_CKBOOL LED::init()
{
    return TRUE;
}

t_CKBOOL LED::display()
{
    Element::display();
    
    return TRUE;
}

t_CKBOOL LED::hide()
{
    Element::hide();
        
    return TRUE;
}

t_CKBOOL LED::destroy()
{
    Element::destroy();
    
    return TRUE;
}

t_CKBOOL LED::set_size( t_CKFLOAT nw, t_CKFLOAT nh )
{
    Element::set_size( nw, nh );
    
    return TRUE;
}

t_CKBOOL LED::set_position( t_CKFLOAT nx, t_CKFLOAT ny )
{
    Element::set_position( nx, ny );
    
    return TRUE;
}

t_CKBOOL LED::set_parent( View * p )
{
    Element::set_parent( p );
    
    return TRUE;
}

t_CKBOOL LED::remove_parent( View * p )
{
    Element::remove_parent( p );
    
    return TRUE;
}

t_CKBOOL LED::light()
{
    return TRUE;
}

t_CKBOOL LED::unlight()
{
    return TRUE;
}

t_CKBOOL LED::set_color( color c )
{    
    return TRUE;
}

LED::color LED::get_color()
{
    return c;
}

t_CKBOOL Mouse::init()
{
    active = FALSE;
    return TRUE;
}

t_CKBOOL Mouse::display()
{
    active = TRUE;
	return TRUE;
}

t_CKBOOL Mouse::hide()
{
    active = FALSE;
	return TRUE;
}

t_CKBOOL Mouse::destroy()
{
	return TRUE;
    // nop
}

t_CKUINT Mouse::x()
{
    return 0;
}

t_CKUINT Mouse::y()
{
    return 0;
}

t_CKUINT Mouse::maxx()
{
    return 0;
}

t_CKUINT Mouse::maxy()
{
    return 0;
}

t_CKDOUBLE Mouse::fx()
{
    return ((t_CKDOUBLE)x())/((t_CKDOUBLE)maxx());
}

t_CKDOUBLE Mouse::fy()
{
    return ((t_CKDOUBLE)y())/((t_CKDOUBLE)maxy());
}


} /* UI */
//} /* miniAudicle */

